@@ -18,6 +18,15 @@ let error =
     QCheck.int
     (fun _ -> raise Error)
 
+let neg_test_failing_as_expected =
+  QCheck.Test.make_neg ~name:"neg test pass (failing as expected)" QCheck.small_int (fun i -> i mod 2 = 0)
+
+let neg_test_unexpected_success =
+  QCheck.Test.make_neg ~name:"neg test unexpected success" QCheck.small_int (fun i -> i + i = i * 2)
+
+let neg_test_error =
+  QCheck.Test.make_neg ~name:"neg fail with error" QCheck.small_int (fun _i -> raise Error)
+
 let simple_qcheck =
   QCheck.Test.make ~name:"fail_check_err_message"
     ~count: 100
@@ -48,13 +57,25 @@ let passing_tree_rev =
     QCheck.(make gen_tree)
     (fun tree -> rev_tree (rev_tree tree) = tree)
 
+let debug_shrink =
+  QCheck.Test.make ~count:10
+    ~name:"debug_shrink"
+    (* we use a very constrained test to have a smaller shrinking tree *)
+    QCheck.(pair (1 -- 3) (1 -- 3))
+    (fun (a, b) -> a = b);;
+
 let () =
   Printexc.record_backtrace true;
   let module A = Alcotest in
   let suite =
     List.map QCheck_alcotest.to_alcotest
-      [ passing; failing; error; simple_qcheck; passing_tree_rev ]
+      [ passing; failing; error;
+        neg_test_failing_as_expected; neg_test_unexpected_success; neg_test_error;
+        simple_qcheck; passing_tree_rev ]
   in
-  A.run "my test" [
-    "suite", suite
-  ]
+  A.run ~show_errors:true "my test" [
+    "suite", suite;
+    "shrinking", [
+      QCheck_alcotest.to_alcotest ~verbose:true ~debug_shrink:(Some stdout) debug_shrink
+    ];
+  ];
