@@ -78,7 +78,7 @@ all rights reserved.
     ]}
 
     More complex and powerful combinators can be found in Gabriel Scherer's
-    {!Generator} module. Its documentation can be found
+    {{:https://github.com/gasche/random-generator}[Generator]} module. Its documentation can be found
     {{:http://gasche.github.io/random-generator/doc/Generator.html } here}.
 *)
 
@@ -169,6 +169,18 @@ module Gen : sig
   (** [map3 f g1 g2 g3] transforms three generators [g1], [g2], and [g3] by applying [f]
       to each triple of generated elements. *)
 
+  val map4 : ('a -> 'b -> 'c -> 'd -> 'e) -> 'a t -> 'b t -> 'c t -> 'd t -> 'e t
+  (** [map4 f g1 g2 g3 g4] transforms four generators [g1], [g2], [g3], and [g4]
+      by applying [f] to each quadruple of generated elements.
+
+      @since 0.25 *)
+
+  val map5 : ('a -> 'b -> 'c -> 'd -> 'e -> 'f) -> 'a t -> 'b t -> 'c t -> 'd t -> 'e t -> 'f t
+  (** [map5 f g1 g2 g3 g4 g5] transforms five generators [g1], [g2], [g3], [g4],
+      and [g5] by applying [f] to each quintuple of generated elements.
+
+      @since 0.25 *)
+
   val map_keep_input : ('a -> 'b) -> 'a t -> ('a * 'b) t
   (** [map_keep_input f g] transforms a generator [g] by applying [f] to each generated element.
       Returns both the generated element from [g] and the output from [f]. *)
@@ -325,6 +337,12 @@ module Gen : sig
   val graft_corners : 'a t -> 'a list -> unit -> 'a t
   (** [graft_corners gen l ()] makes a new generator that enumerates
       the corner cases in [l] and then behaves like [g].
+
+      Note that [graft_corners gen l ()] is stateful, meaning that once the
+      elements of [l] have been emitted, subsequent calls will not reproduce
+      them. It is therefore recommended that separate tests each use a fresh
+      generator.
+
       @since 0.6 *)
 
   val int_pos_corners : int list
@@ -415,7 +433,7 @@ module Gen : sig
   val bytes : ?gen:char t -> bytes t
   (** Builds a bytes generator. Bytes size is generated by {!nat}.
       Accepts an optional character generator (the default is {!char}).
-      See also {!bytes_of} and {!bytes_readable} for versions without
+      See also {!bytes_of} and {!bytes_printable} for versions without
       optional parameters.
       @since 0.20 *)
 
@@ -640,7 +658,7 @@ module Gen : sig
   (** {{: https://ocaml.org/manual/bindingops.html} Binding operator} alias for {!pair}. *)
 
   val ( let* ) : 'a t -> ('a -> 'b t) -> 'b t
-  (** {{: https://ocaml.org/manual/bindingops.html} Binding operator} alias for {!bind}. *)
+  (** {{: https://ocaml.org/manual/bindingops.html} Binding operator} alias for {!(>>=)}. *)
 
   val ( and* ) : 'a t -> 'b t -> ('a * 'b) t
   (** {{: https://ocaml.org/manual/bindingops.html} Binding operator} alias for {!pair}. *)
@@ -1144,7 +1162,18 @@ val pos_int : int arbitrary
 
 val small_int_corners : unit -> int arbitrary
 (** As [small_int], but each newly created generator starts with
-    a list of corner cases before falling back on random generation. *)
+    a list of corner cases before falling back on random generation.
+
+    Note that [small_int_corners ()] is stateful, meaning that once the list of
+    corner cases has been emitted, subsequent calls will not reproduce them.
+    As a consequence, in the following example, the first test fails with a
+    counter example, whereas the second rerun does not:
+    {[
+      let gen = QCheck.small_int_corners ()
+      let t = QCheck.Test.make ~name:"never max_int" gen (fun i -> i <> max_int)
+      let _ = QCheck_base_runner.run_tests ~verbose:true [t;t]
+    ]}
+ *)
 
 val neg_int : int arbitrary
 (** Negative int generator (0 included, see {!Gen.neg_int}).
@@ -1400,7 +1429,7 @@ val tup9 :
 
 val choose : 'a arbitrary list -> 'a arbitrary
 (** Choose among the given list of generators. The list must not
-    be empty; if it is Invalid_argument is raised. *)
+    be empty; if it is [Invalid_argument] is raised. *)
 
 val oneofl : ?print:'a Print.t -> ?collect:('a -> string) ->
   'a list -> 'a arbitrary
@@ -1878,7 +1907,7 @@ module Tuple : sig
   val nil : unit t
   val cons : 'a -> 'b t -> ('a * 'b) t
 
-  (** How to observe a  {!'a t} *)
+  (** How to observe a  {{!t}['a t]} *)
   type 'a obs
 
   val o_nil : unit obs
@@ -1889,7 +1918,7 @@ module Tuple : sig
     (** Alias to {!cons}. *)
 
     val (@->) : 'a Observable.t -> 'b obs -> ('a * 'b) obs
-    (** Alias to {!B_cons}. *)
+    (** Alias to {!o_cons}. *)
   end
 
   include module type of Infix
