@@ -330,10 +330,20 @@ module Gen = struct
   let liftA3 (f : 'a -> 'b -> 'c -> 'd) (a : 'a t) (b : 'b t) (c : 'c t) : 'd t =
     (a >|= f) <*> b <*> c
 
+  let liftA4 (f : 'a -> 'b -> 'c -> 'd -> 'e) (a : 'a t) (b : 'b t) (c : 'c t) (d : 'd t) : 'e t =
+    (a >|= f) <*> b <*> c <*> d
+
+  let liftA5 (f : 'a -> 'b -> 'c -> 'd -> 'e -> 'f) (a : 'a t) (b : 'b t) (c : 'c t) (d : 'd t) (e : 'e t) : 'f t =
+    (a >|= f) <*> b <*> c <*> d <*> e
+
   let map2 = liftA2
 
   let map3 = liftA3
 
+  let map4 = liftA4
+
+  let map5 = liftA5
+
   let return = pure
 
   let bind (gen : 'a t) (f : 'a -> ('b t)) : 'b t = fun st ->
@@ -884,7 +894,7 @@ module Gen = struct
     make_primitive
       ~gen:(fun st -> gen st |> Tree.root)
       ~shrink
-  
+
   let no_shrink (gen: 'a t) : 'a t = set_shrink (fun _ -> Seq.empty) gen
 
   let (let+) = (>|=)
@@ -1868,6 +1878,7 @@ module Test = struct
     state.step state.test.name state.test input Failure;
     state.cur_max_fail <- state.cur_max_fail - 1;
     R.fail state.res ~steps ~msg_l input;
+    if state.cur_max_fail > 0 then CR_continue else
     CR_yield state.res
 
   (* [check_state state] applies [state.test] repeatedly ([iter] times)
@@ -2030,17 +2041,24 @@ module Test = struct
   let stat_max_lines = 20 (* maximum number of lines for a histogram *)
 
   let print_stat ((name,_), tbl) =
-    let avg = ref 0. in
+    let neg_avg_summands = ref [] in
+    let pos_avg_summands = ref [] in
     let num = ref 0 in
     let min_idx, max_idx =
       Hashtbl.fold
         (fun i res (m1,m2) ->
-           avg := !avg +. float_of_int (i * res);
+           let avg_summand = float_of_int (i * res) in
+           if avg_summand < 0.
+           then neg_avg_summands := avg_summand::!neg_avg_summands
+             else pos_avg_summands := avg_summand::!pos_avg_summands;
            num := !num + res;
            min i m1, max i m2)
         tbl (max_int,min_int)
     in
-    (* compute average *)
+    (* compute average, summing positive/negative separately by order of magnitude *)
+    let neg_avg_sums = List.sort Float.compare !neg_avg_summands |> fun xs -> List.fold_right (+.) xs 0. in
+    let pos_avg_sums = List.sort Float.compare !pos_avg_summands |> List.fold_left (+.) 0. in
+    let avg = ref (neg_avg_sums +. pos_avg_sums) in
     if !num > 0 then (
       avg := !avg /. float_of_int !num
     );
@@ -2048,8 +2066,10 @@ module Test = struct
        https://en.wikipedia.org/wiki/Standard_deviation *)
     let stddev =
       Hashtbl.fold
-        (fun i res m -> m +. (float_of_int i -. !avg) ** 2. *. float_of_int res)
-        tbl 0.
+        (fun i res acc -> float_of_int res *. ((float_of_int i -. !avg) ** 2.) :: acc)
+        tbl []
+      |> List.sort Float.compare (* add summands in increasing order to preserve precision *)
+      |> List.fold_left (+.) 0.
       |> (fun s -> if !num>0 then s /. float_of_int !num else s)
       |> sqrt
     in
@@ -2090,10 +2110,17 @@ module Test = struct
          max_val := max !max_val new_count) tbl;
     (* print entries of the table, sorted by increasing index *)
     let out = Buffer.create 128 in
+    (* Windows workaround to avoid annoying exponent zero such as "1.859e+018" *)
+    let cut_exp_zero s =
+      match String.split_on_char '+' s with
+      | [signif;exponent] -> Printf.sprintf "%s+%i" signif (int_of_string exponent)
+      | _ -> failwith "cut_exp_zero failed to parse scientific notation " ^ s in
+    let fmt_float f =
+      if f > 1e7 || f < -1e7 then cut_exp_zero (Printf.sprintf "%.3e" f) else Printf.sprintf "%.2f" f in
     Printf.bprintf out "stats %s:\n" name;
     Printf.bprintf out
-      "  num: %d, avg: %.2f, stddev: %.2f, median %d, min %d, max %d\n"
-      !num !avg stddev !median min_idx max_idx;
+      "  num: %d, avg: %s, stddev: %s, median %d, min %d, max %d\n"
+      !num (fmt_float !avg) (fmt_float stddev) !median min_idx max_idx;
     let indwidth =
       let str_width i = String.length (Printf.sprintf "%d" i) in
       List.map str_width [min_idx; max_idx; min_idx + bucket_size * hist_size] |> List.fold_left max min_int in
