@@ -60,7 +60,17 @@ let set_seed_ ~colors s =
 (* time of last printed message. Useful for rate limiting in verbose mode *)
 let last_msg = ref 0.
 
-let time_between_msg = ref 0.1
+let time_between_msg =
+  let env_var = "QCHECK_MSG_INTERVAL" in
+  let default_interval = 0.1 in
+  let interval = match Sys.getenv_opt env_var with
+    | None -> default_interval
+    | Some f ->
+      match float_of_string_opt f with
+      | None -> invalid_arg (env_var ^ " must be a float")
+      | Some i -> i in
+  if interval < 0. then invalid_arg (env_var ^ " must be >= 0 but value is " ^ string_of_float interval);
+  ref interval
 
 let get_time_between_msg () = !time_between_msg
 
@@ -135,9 +145,14 @@ module Raw = struct
     if print_res then (
       (* even if [not verbose], print errors *)
       match R.get_state result with
-        | R.Success -> ()
+        | R.Success ->
+          if not (T.get_positive cell)
+          then
+            print.fail "%snegative test '%s' succeeded unexpectedly\n" reset_line name;
         | R.Failed {instances=l} ->
-          print.fail "%s%s\n" reset_line (T.print_fail cell name l);
+          if T.get_positive cell
+          then print.fail "%s%s\n" reset_line (T.print_fail cell name l)
+          else print.info "%s%s\n" reset_line (T.print_expected_failure cell l)
         | R.Failed_other {msg} ->
           print.fail "%s%s\n" reset_line (T.print_fail_other name ~msg);
         | R.Error {instance; exn; backtrace} ->
@@ -175,7 +190,7 @@ module Raw = struct
           ; "--debug-shrink-list", Arg.String set_debug_shrink_list, " filter test to debug shrinking on"
           ]
       ) in
-    Arg.parse_argv argv options (fun _ ->()) "run qtest suite";
+    Arg.parse_argv argv options (fun _ ->()) "run QCheck test suite";
     let cli_rand = setup_random_state_ ~colors:!colors () in
     { cli_verbose=verbose(); cli_long_tests=long_tests(); cli_rand;
       cli_print_list= !print_list; cli_slow_test= !slow;
@@ -219,28 +234,14 @@ let debug_shrinking_counter_example cell out x =
   | None -> Printf.fprintf out "<no printer provided>"
   | Some print -> Printf.fprintf out "%s" (print x)
 
-let debug_shrinking_choices_aux ~colors out name i cell x =
+let debug_shrinking_choices ~colors ~out ~name cell ~step x =
   Printf.fprintf out "\n~~~ %a %s\n\n"
     (Color.pp_str_c ~colors `Cyan) "Shrink" (String.make 69 '~');
   Printf.fprintf out
     "Test %s successfully shrunk counter example (step %d) to:\n\n%a\n%!"
-    name i
+    name step
     (debug_shrinking_counter_example cell) x
 
-let debug_shrinking_choices
-    ~colors ~debug_shrink ~debug_shrink_list name cell i x =
-  match debug_shrink with
-  | None -> ()
-  | Some out ->
-    begin match debug_shrink_list with
-      | [] ->
-        debug_shrinking_choices_aux ~colors out name i cell x
-      | l when List.mem name l ->
-        debug_shrinking_choices_aux ~colors out name i cell x
-      | _ -> ()
-    end
-
-
 let default_handler
   ~colors ~debug_shrink ~debug_shrink_list
   ~size ~out ~verbose c =
@@ -256,9 +257,20 @@ let default_handler
     in
     (* debug shrinking choices *)
     begin match r with
-      | QCheck2.Test.Shrunk (i, x) ->
-          debug_shrinking_choices
-          ~colors ~debug_shrink ~debug_shrink_list name cell i x
+      | QCheck2.Test.Shrunk (step, x) ->
+        begin match debug_shrink with
+          | None -> ()
+          | Some out ->
+            let go =
+              match debug_shrink_list with
+              | [] -> true
+              | test_list -> List.mem name test_list
+            in
+            if not go then ()
+            else
+              debug_shrinking_choices
+                ~colors ~out ~name cell ~step x
+        end
       | _ ->
         ()
     end;
@@ -289,8 +301,11 @@ let step ~colors ~size ~out ~verbose c name _ _ r =
       (if colors then Color.reset_line else "\n") (pp_counter ~size) c name
   )
 
-let callback ~size ~out ~verbose ~colors c name _ r =
-  let pass = QCheck2.TestResult.is_success r in
+let callback ~size ~out ~verbose ~colors c name cell r =
+  let pass =
+    if QCheck2.Test.get_positive cell
+    then QCheck2.TestResult.is_success r
+    else QCheck2.TestResult.is_failed r in
   let color = if pass then `Green else `Red in
   if verbose then (
     Printf.fprintf out "%s[%a] %a %s\n%!"
@@ -359,6 +374,13 @@ let print_fail_other ~colors out cell msg =
   Printf.fprintf out "\n--- %a %s\n\n" (Color.pp_str_c ~colors `Red) "Failure" (String.make 68 '-');
   Printf.fprintf out "Test %s failed:\n\n%s\n%!" (QCheck2.Test.get_name cell) msg
 
+let print_expected_failure ~colors out cell c_ex =
+  Printf.fprintf out "\n--- %a %s\n\n" (Color.pp_str_c ~colors `Blue) "Info" (String.make 71 '-');
+  Printf.fprintf out "Negative test %s failed as expected (%d shrink steps):\n\n%s\n%!"
+    (QCheck2.Test.get_name cell) c_ex.QCheck2.TestResult.shrink_steps
+    (print_inst cell c_ex.QCheck2.TestResult.instance);
+  print_messages ~colors out cell c_ex.QCheck2.TestResult.msg_l
+
 let print_error ~colors out cell c_ex exn bt =
   Printf.fprintf out "\n=== %a %s\n\n" (Color.pp_str_c ~colors `Red) "Error" (String.make 70 '=');
   Printf.fprintf out "Test %s errored on (%d shrink steps):\n\n%s\n\nexception %s\n%s\n%!"
@@ -409,17 +431,24 @@ let run_tests
   let res = List.map aux_map l in
   let aux_fold (total, fail, error, warns) (Res (cell, r)) =
     let warns = warns + List.length (R.get_warnings r) in
-    let acc = match R.get_state r with
-      | R.Success ->
+    let acc = match R.get_state r, QCheck2.Test.get_positive cell with
+      | R.Success, true ->
         print_success ~colors out cell r;
         (total + 1, fail, error, warns)
-      | R.Failed {instances=l} ->
-        List.iter (print_fail ~colors out cell) l;
-        (total + 1, fail + 1, error, warns)
-      | R.Failed_other {msg} ->
+      | R.Success, false ->
+        let msg = Printf.sprintf "Negative test %s succeeded but was expected to fail" (QCheck2.Test.get_name cell) in
         print_fail_other ~colors out cell msg;
         (total + 1, fail + 1, error, warns)
-      | R.Error {instance=c_ex; exn; backtrace=bt} ->
+      | R.Failed {instances=l}, true ->
+        List.iter (print_fail ~colors out cell) l;
+        (total + 1, fail + 1, error, warns)
+      | R.Failed {instances=l}, false ->
+        if verbose then List.iter (print_expected_failure ~colors out cell) l;
+        (total + 1, fail, error, warns)
+      | R.Failed_other {msg}, _ ->  (* Failed_other is also considered a failure *)
+        print_fail_other ~colors out cell msg;
+        (total + 1, fail + 1, error, warns)
+      | R.Error {instance=c_ex; exn; backtrace=bt}, _ -> (* Error is always considered a failure *)
         print_error ~colors out cell c_ex exn bt;
         (total + 1, fail, error + 1, warns)
     in
