@@ -51,6 +51,10 @@ module Overall = struct
     Test.make ~name:"should_fail_sort_id" ~count:10 ~print:Print.(list int)
       Gen.(small_list small_int) (fun l -> l = List.sort compare l)
 
+  let max_fail =
+    Test.make ~name:"max_fail" ~count:1000 ~max_fail:3 ~print:Print.(list int)
+      Gen.(list small_nat) (fun l -> l = List.rev l)
+
   exception Error
 
   let error =
@@ -142,6 +146,7 @@ module Overall = struct
   let tests = [
     passing;
     failing;
+    max_fail;
     error;
     collect;
     stats;
